/******************************************************************************
;  *       @ͺ                 : MC51F7424
;  *       @             : 2023.02.01
;  *       @˾/            : SINOMCU.COM-AE
;  *       @΢           : http://www.sinomcu.com/
;  *       @Ȩ                 : 2022 ΢ Ȩ.
;  *----------------------ժҪ---------------------------------
;  * Cacl_16mul16() 16bit*16bit,ʹMCUӲ.
;  * Cacl_32div16() 32bit/16bit,ʹMCUӲ.
;  *               ʹȫֱ(ٺ÷ֵֽλ).
;  * Cacl_32div16_gvar() 32bit/16bit,MCUӲ,޴,ȫֱ.
******************************************************************************/

#include "user_config.h"
#include "GPIO.h"

#define Start_HWmul_uintCacl()   MDUCR = 0x01    /*˷ ޷*/
#define Start_HWdiv_uintCacl()   MDUCR = 0x41    /* ޷*/
#define WAIT_HWdiv_finish()      NOP(8)
// #define WAIT_HWdiv_finish()      while(MDUCR &0x01) {;}

/**
  * @brief  calculation 16bit multiplied 16bit
  * @param  
  *     uint16_t mulA:multiplicand
  *     uint16_t mulB:multiplier
  * @retval uint32_t:multiplied result
  */ 
uint32_t Cacl_16mul16(uint16_t data mulA,uint16_t data mulB) {
#if(defined(EN_HW_MUL_DIV) && EN_HW_MUL_DIV )
#else
  uint32_t Cacl_result=0;
#endif
  TEST_PIN_HIGH();      /*GPIO output high for test calculation time*/
#if(defined(EN_HW_MUL_DIV) && EN_HW_MUL_DIV )  
  MDUDRA1 = mulA>>8;    /*multiplicand high 8bit*/
  MDUDRA0 = mulA;       /*multiplicand low 8bit*/
  MDUDRB1 = mulB>>8;    /*multiplier high 8bit*/
  MDUDRB0 = mulB;       /*multiplier low 8bit*/
  Start_HWmul_uintCacl();
#pragma ASM            
  MOV R7,MDUDRA0        /*return value low byte*/
  MOV R6,MDUDRA1        /*return value middle low byte*/
  MOV R5,MDUDRA2        /*return value middle hgih byte*/
  MOV R4,MDUDRA3        /*return value hgih byte*/
#pragma ENDASM
  TEST_PIN_LOW();
#else
  Cacl_result =(uint32_t)mulA * mulB;
  TEST_PIN_LOW(); 
  return Cacl_result;    
#endif
}

/*global division remainder*/
uint16_t g_div_remainder=0;

/**
  * @brief  calculation 32bit division 16bit
  * @param  
  *     uint32_t divA:dividend
  *     uint16_t divB:divisor
  * @retval uint32_t:division result
  * @note
  *     g_div_remainder(global variable) will be change by this function 
  */ 
uint32_t Cacl_32div16(uint32_t divA,uint16_t divB) {
#if(defined(EN_HW_MUL_DIV) && EN_HW_MUL_DIV)
#else
  uint32_t Cacl_result=0;
#endif
  TEST_PIN_HIGH();      /*GPIO output high for test calculation time*/
#if(defined(EN_HW_MUL_DIV) && EN_HW_MUL_DIV) 
//   MDUDRA3 = divA>>24;   /*dividend hgih byte*/
//   MDUDRA2 = divA>>16;   /*dividend middle hgih 8bit*/
//   MDUDRA1 = divA>>8;    /*dividend middle low 8bit*/
//   MDUDRA0 = divA;       /*dividend low 8bit*/
#pragma ASM  
  MOV MDUDRA0,R7;      /*dividend hgih byte*/
  MOV MDUDRA1,R6;      /*dividend middle hgih 8bit*/
  MOV MDUDRA2,R5;      /*dividend middle low 8bit*/
  MOV MDUDRA3,R4;      /*dividend low 8bit*/
#pragma ENDASM

  MDUDRB1 = divB>>8;    /*divisor high 8bit*/
  MDUDRB0 = divB;       /*divisor low 8bit*/
  Start_HWdiv_uintCacl();
  WAIT_HWdiv_finish();
//   g_div_remainder =MDUDRB1;  /*division remainder*/
//   g_div_remainder =g_div_remainder<<8;
//   g_div_remainder |=MDUDRB0;
#pragma ASM  
  MOV g_div_remainder,MDUDRB1
  MOV g_div_remainder+1,MDUDRB0 
  MOV R7,MDUDRA0        /*return value low byte*/
  MOV R6,MDUDRA1        /*return value middle low byte*/
  MOV R5,MDUDRA2        /*return value middle hgih byte*/
  MOV R4,MDUDRA3        /*return value hgih byte*/
#pragma ENDASM
  TEST_PIN_LOW();
#else
  Cacl_result = divA / divB;
  TEST_PIN_LOW();
  return Cacl_result;  
#endif
}

#if(defined(EN_HW_MUL_DIV) && EN_HW_MUL_DIV)
uint32_t data g_divA=0;
uint16_t data g_divB=0;
uint32_t data g_div_result=0;

/**
  * @brief  calculation 32bit division 16bit,global variable
  * @param  
  * @retval 
  * @note
  *     g_divA(global variable):dividend
  *     g_divB(global variable):divisor
  *     g_div_result,g_div_remainder(global variable)
  */ 
void Cacl_32div16_gvar(void) {    
  TEST_PIN_HIGH();      /*GPIO output high for test calculation time*/ 
#pragma ASM               
  MOV MDUDRA3,g_divA    /*dividend hgih byte*/
  MOV MDUDRA2,g_divA+1  /*dividend middle hgih 8bit*/
  MOV MDUDRA1,g_divA+2  /*dividend middle low 8bit*/
  MOV MDUDRA0,g_divA+3  /*dividend low 8bit*/   

  MOV MDUDRB1,g_divB    /*divisor high 8bit*/
  MOV MDUDRB0,g_divB+1  /*divisor low 8bit*/
#pragma ENDASM
  MDUCR    = 0x41;
  WAIT_HWdiv_finish();
 #pragma ASM               
  MOV g_div_result,MDUDRA3        /*result value hgih byte*/
  MOV g_div_result+1,MDUDRA2      /*result value middle hgih byte*/
  MOV g_div_result+2,MDUDRA1      /*result value middle low byte*/
  MOV g_div_result+3,MDUDRA0      /*result value low byte*/
  MOV g_div_remainder,MDUDRB1     /*remainder hgih byte*/  
  MOV g_div_remainder+1,MDUDRB0   /*remainder low byte*/   
#pragma ENDASM       
  TEST_PIN_LOW();   
}
#endif

/******************************** END OF FILE *********************************/